package expressionTree.interfaces;

import java.io.PrintWriter;

/** Each object implementing this interface represents a boolean expression.
 * 
 * @author Theodore Norvell
 *
 */
public interface ExpressionNodeI {
	/**
	 * Evaluate the expression given a particular environment.
	 * <p>
	 * Precondition: All variables used in the expression are
	 * guaranteed to be in the environment.
	 * <p>
	 * Changes: Nothing. This is an accessor.
	 * <p>
	 * Postcondition: The result of evaluate will be the value
	 * of the expression represented by this node and its descendants
	 * in the given environment.
	 * 
	 */
	public boolean evaluate( Environment env ) ; 
	
	/** Print the expression to the given PrintWriter
	 * <p>
	 * Changes: Only the PrintWriter.
	 * <p>
	 * Binary operators will be printed surrounded by parentheses.
	 */
	public void printTo( PrintWriter p) ; 
}
